@@ -8,6 +8,7 @@ def __(key):
 NAME = "Text Mining"
 
 # Category metadata.
+import sysconfig
 
 # Category icon show in the menu
 ICON = "icons/category.svg"
@@ -24,13 +25,17 @@ DESCRIPTION = __("package_desc")
 WIDGET_HELP_PATH = (
     # Used for development.
     # You still need to build help pages using
-    # make htmlhelp
-    # inside doc folder
-    ("{DEVELOP_ROOT}/doc/_build/htmlhelp/index.html", None),
+    # `make html` inside doc folder
+    ("{DEVELOP_ROOT}/doc/_build/html/index.html", None),
+
+    # Documentation included in wheel
+    # Correct DATA_FILES entry is needed in setup.py and documentation has to be built
+    # before the wheel is created.
+    ("{}/help/orange3-text/index.html".format(sysconfig.get_path("data")), None),
 
     # Online documentation url, used when the local documentation is available.
     # Url should point to a page with a section Widgets. This section should
     # includes links to documentation pages of each widget. Matching is
     # performed by comparing link caption to widget name.
-    ("https://orange3-text.readthedocs.io/en/latest/", "")
+    ("https://orange3-text.readthedocs.io/en/latest/", ""),
 )
