@@ -2,7 +2,7 @@
 from collections import Counter
 from itertools import cycle
 from math import pi as PI
-from typing import Dict, List, Optional, Tuple
+from typing import Dict, List, Tuple
 
 import numpy as np
 from AnyQt import QtCore
@@ -28,7 +28,7 @@ COLORS = ["#da1", "#629", "#787"]
 GRAY_COLORS = ["#000", "#444", "#777", "#aaa"]
 TOPIC_COLORS = ["#ff6600", "#00cc00"]  # [negative topic, positive topic]
 GRAY_TOPIC_COLORS = ["#000", "#aaa"]  # [negative topic, positive topic]
-TILT_VALUES = ("no", "30°", "45°", "60°")
+TILT_VALUES = (__("value_no"), "30°", "45°", "60°")
 
 N_BEST_PLOTTED = 200
 
@@ -64,7 +64,7 @@ def count_words(data: Corpus, state: TaskState) -> Tuple[Counter, bool]:
     Reports counts as a counter and boolean that tell whether the data were
     retrieved on bag of words basis.
     """
-    state.set_status("Calculating...")
+    state.set_status(__("state.calculate"))
     state.set_progress_value(0)
     bow_counts = _bow_words(data)
     state.set_progress_value(0.5)
@@ -117,7 +117,7 @@ class OWWordCloud(OWWidget, ConcurrentWidgetMixin):
         topic = Input("Topic", Topic, label=i18n.t("text.common.topic"))
 
     class Outputs:
-        corpus = Output("Corpus", Corpus, label=i18n.t("text.common.corpus"))
+        corpus = Output("Corpus", Corpus, default=True, label=i18n.t("text.common.corpus"))
         selected_words = Output("Selected Words", Topic, dynamic=False, label=i18n.t("text.common.select_word"))
         word_counts = Output("Word Counts", Table, label=i18n.t("text.common.word_count"))
 
@@ -151,8 +151,6 @@ class OWWordCloud(OWWidget, ConcurrentWidgetMixin):
         self.combined_size_length = None
         self._create_layout()
         self.on_corpus_change(None)
-        self.update_input_summary()
-        self.update_output_summary(None, None)
 
     def _new_webview(self):
         HTML = """
@@ -418,7 +416,7 @@ span.selected {color:red !important}
                 metas=[StringVariable("Word")],
             )
             wc_table = Table.from_numpy(domain, X=w_count, metas=words)
-            wc_table.name = "Word Counts"
+            wc_table.name = __("table_name")
         self.Outputs.word_counts.send(wc_table)
 
     @Inputs.corpus
@@ -447,7 +445,6 @@ span.selected {color:red !important}
             self._apply_corpus()
         else:
             self.clear()
-            self.update_input_summary()
             return
 
         self.Warning.topic_precedence(
@@ -457,7 +454,6 @@ span.selected {color:red !important}
             if self.selected_words:
                 self.update_selection(self.selected_words)
         self.commit()
-        self.update_input_summary()
 
     def clear(self):
         self._new_webview()
@@ -495,52 +491,8 @@ span.selected {color:red !important}
                 X=np.empty((len(words), 0)),
                 metas=np.c_[words].astype(object),
             )
-            topic.name = "Selected Words"
+            topic.name = __("topic_name")
         self.Outputs.selected_words.send(topic)
-        self.update_output_summary(
-            len(out) if out is not None else None,
-            len(topic) if topic is not None else None,
-        )
-
-    def update_input_summary(self) -> None:
-        if self.corpus is None and self.topic is None:
-            self.info.set_input_summary(self.info.NoInput)
-        else:
-            input_string = ""
-            input_numbers = ""
-            if self.corpus is not None:
-                input_string += (
-                    __("msg.data_info").format(len(self.corpus), len(self.corpus_counter))
-                )
-                input_numbers += str(len(self.corpus_counter))
-            if self.topic is not None:
-                input_string += __("msg.word_in_topic").format(self.n_topic_words)
-                input_numbers += (
-                    f"{' | ' if input_numbers else ''}" f"{self.n_topic_words}"
-                )
-            self.info.set_input_summary(input_numbers, input_string)
-
-    def update_output_summary(
-            self, cor_output_len: Optional[int], n_selected: Optional[int]
-    ) -> None:
-        if (
-                cor_output_len is None
-                and n_selected is None
-                and (self.corpus_counter is None or len(self.corpus_counter) == 0)
-        ):
-            self.info.set_output_summary(self.info.NoOutput)
-        else:
-            cc_len = (
-                len(self.corpus_counter)
-                if self.corpus_counter is not None
-                else 0
-            )
-            input_numbers = f"{cor_output_len or 0} | {n_selected or 0} | " \
-                            f"{cc_len}"
-            input_string = (
-                __("msg.datas_info").format((cor_output_len or 0), (n_selected or 0), (cc_len))
-            )
-            self.info.set_output_summary(input_numbers, input_string)
 
     def send_report(self):
         if self.webview:
